@@ -87,9 +87,11 @@ import org.apache.activemq.management.JMSStatsImpl;
 import org.apache.activemq.management.StatsCapable;
 import org.apache.activemq.management.StatsImpl;
 import org.apache.activemq.state.CommandVisitorAdapter;
+import org.apache.activemq.thread.Scheduler;
 import org.apache.activemq.thread.TaskRunnerFactory;
 import org.apache.activemq.transport.Transport;
 import org.apache.activemq.transport.TransportListener;
+import org.apache.activemq.transport.failover.FailoverTransport;
 import org.apache.activemq.util.IdGenerator;
 import org.apache.activemq.util.IntrospectionSupport;
 import org.apache.activemq.util.JMSExceptionSupport;
@@ -113,7 +115,7 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
     protected boolean alwaysSessionAsync = true;
 
     private TaskRunnerFactory sessionTaskRunner;
-    private final ThreadPoolExecutor asyncConnectionThread;
+    private final ThreadPoolExecutor executor;
 
     // Connection state variables
     private final ConnectionInfo info;
@@ -145,6 +147,7 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
     private long warnAboutUnstartedConnectionTimeout = 500L;
     private int sendTimeout =0;
     private boolean sendAcksAsync=true;
+    private boolean checkForDuplicates = true;
 
     private final Transport transport;
     private final IdGenerator clientIdGenerator;
@@ -187,6 +190,8 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
     private boolean useDedicatedTaskRunner;
     protected volatile CountDownLatch transportInterruptionProcessingComplete;
     private long consumerFailoverRedeliveryWaitPeriod;
+    private final Scheduler scheduler;
+    private boolean messagePrioritySupported=true;
 
     /**
      * Construct an <code>ActiveMQConnection</code>
@@ -203,17 +208,18 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
 
         // Configure a single threaded executor who's core thread can timeout if
         // idle
-        asyncConnectionThread = new ThreadPoolExecutor(1, 1, 5, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(), new ThreadFactory() {
+        executor = new ThreadPoolExecutor(1, 1, 5, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(), new ThreadFactory() {
             public Thread newThread(Runnable r) {
-                Thread thread = new Thread(r, "ActiveMQ Connection Worker: " + transport);
+                Thread thread = new Thread(r, "ActiveMQ Connection Executor: " + transport);
                 thread.setDaemon(true);
                 return thread;
             }
         });
         // asyncConnectionThread.allowCoreThreadTimeOut(true);
-
-        this.info = new ConnectionInfo(new ConnectionId(CONNECTION_ID_GENERATOR.generateId()));
+        String uniqueId = CONNECTION_ID_GENERATOR.generateId();
+        this.info = new ConnectionInfo(new ConnectionId(uniqueId));
         this.info.setManageable(true);
+        this.info.setFaultTolerant(transport.isFaultTolerant());
         this.connectionSessionId = new SessionId(info.getConnectionId(), -1);
 
         this.transport.setTransportListener(this);
@@ -222,6 +228,8 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
         this.factoryStats.addConnection(this);
         this.timeCreated = System.currentTimeMillis();
         this.connectionAudit.setCheckForDuplicates(transport.isFaultTolerant());
+        this.scheduler = new Scheduler("ActiveMQConnection["+uniqueId+"] Scheduler");
+        this.scheduler.start();
     }
 
     protected void setUserName(String userName) {
@@ -607,6 +615,14 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
                         advisoryConsumer.dispose();
                         advisoryConsumer = null;
                     }
+                    if (this.scheduler != null) {
+                        try {
+                            this.scheduler.stop();
+                        } catch (Exception e) {
+                            JMSException ex =  JMSExceptionSupport.create(e);
+                            throw ex;
+                        }
+                    }
 
                     long lastDeliveredSequenceId = 0;
                     for (Iterator<ActiveMQSession> i = this.sessions.iterator(); i.hasNext();) {
@@ -627,6 +643,14 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
                         c.dispose();
                     }
 
+                    // As TemporaryQueue and TemporaryTopic instances are bound
+                    // to a connection we should just delete them after the connection
+                    // is closed to free up memory
+                    for (Iterator<ActiveMQTempDestination> i = this.activeTempDestinations.values().iterator(); i.hasNext();) {
+                        ActiveMQTempDestination c = i.next();
+                        c.delete();
+                    }
+                    
                     if (isConnectionInfoSentToBroker) {
                         // If we announced ourselfs to the broker.. Try to let
                         // the broker
@@ -654,8 +678,8 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
             }
         } finally {
             try {
-                if (asyncConnectionThread != null){
-                    asyncConnectionThread.shutdown();
+                if (executor != null){
+                    executor.shutdown();
                 }
             }catch(Throwable e) {
                 LOG.error("Error shutting down thread pool " + e,e);
@@ -1302,7 +1326,9 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
 	private Response doSyncSendPacket(Command command, int timeout)
 			throws JMSException {
 		try {
-		    Response response = (Response)this.transport.request(command, timeout);
+		    Response response = (Response) (timeout > 0
+                    ? this.transport.request(command, timeout) 
+                    : this.transport.request(command));
 		    if (response != null && response.isException()) {
 		        ExceptionResponse er = (ExceptionResponse)response;
 		        if (er.getException() instanceof JMSException) {
@@ -1418,6 +1444,20 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
     public void setAlwaysSyncSend(boolean alwaysSyncSend) {
         this.alwaysSyncSend = alwaysSyncSend;
     }
+    
+    /**
+     * @return the messagePrioritySupported
+     */
+    public boolean isMessagePrioritySupported() {
+        return this.messagePrioritySupported;
+    }
+
+    /**
+     * @param messagePrioritySupported the messagePrioritySupported to set
+     */
+    public void setMessagePrioritySupported(boolean messagePrioritySupported) {
+        this.messagePrioritySupported = messagePrioritySupported;
+    }
 
     /**
      * Cleans up this connection so that it's state is as if the connection was
@@ -1674,24 +1714,7 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
                 command.visit(new CommandVisitorAdapter() {
                     @Override
                     public Response processMessageDispatch(MessageDispatch md) throws Exception {
-                        if (willBlockWaitingForTransportInterruptionProcessing()) {
-                            // delivery new messages async till interruption processing is complete, on trunk consumers
-                            // are recreated as pull only pending interruption processing
-                            if (LOG.isTraceEnabled()) {
-                                LOG.trace("async delivery pending Interruption processing for: " + md.getMessage().getMessageId());
-                            }
-                            asyncConnectionThread.execute(new Runnable() {
-                                public void run() {
-                                    try {
-                                        waitForTransportInterruptionProcessing();
-                                        onCommand(command);
-                                    } catch (Exception e) {
-                                        onClientInternalException(e);
-                                    }
-                                }
-                            });
-                            return null;
-                        }
+                        waitForTransportInterruptionProcessingToComplete();
                         ActiveMQDispatcher dispatcher = dispatchers.get(md.getConsumerId());
                         if (dispatcher != null) {
                             // Copy in case a embedded broker is dispatching via
@@ -1734,7 +1757,7 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
 
                     @Override
                     public Response processConnectionError(final ConnectionError error) throws Exception {
-                        asyncConnectionThread.execute(new Runnable() {
+                        executor.execute(new Runnable() {
                             public void run() {
                                 onAsyncException(error.getException());
                             }
@@ -1794,7 +1817,7 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
     public void onClientInternalException(final Throwable error) {
         if ( !closed.get() && !closing.get() ) {
             if ( this.clientInternalExceptionListener != null ) {
-                asyncConnectionThread.execute(new Runnable() {
+                executor.execute(new Runnable() {
                     public void run() {
                         ActiveMQConnection.this.clientInternalExceptionListener.onException(error);
                     }
@@ -1819,7 +1842,7 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
                 }
                 final JMSException e = (JMSException)error;
 
-                asyncConnectionThread.execute(new Runnable() {
+                executor.execute(new Runnable() {
                     public void run() {
                         ActiveMQConnection.this.exceptionListener.onException(e);
                     }
@@ -1834,7 +1857,7 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
     public void onException(final IOException error) {
 		onAsyncException(error);
 		if (!closing.get() && !closed.get()) {
-			asyncConnectionThread.execute(new Runnable() {
+			executor.execute(new Runnable() {
 				public void run() {
 					transportFailed(error);
 					ServiceSupport.dispose(ActiveMQConnection.this.transport);
@@ -1855,14 +1878,21 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
 	}
 
     public void transportInterupted() {
-        transportInterruptionProcessingComplete = new CountDownLatch(dispatchers.size() - (advisoryConsumer != null ? 1:0));
+        this.transportInterruptionProcessingComplete = new CountDownLatch(dispatchers.size() - (advisoryConsumer != null ? 1:0));
         if (LOG.isDebugEnabled()) {
             LOG.debug("transport interrupted, dispatchers: " + transportInterruptionProcessingComplete.getCount());
         }
+        signalInterruptionProcessingNeeded();
+
         for (Iterator<ActiveMQSession> i = this.sessions.iterator(); i.hasNext();) {
             ActiveMQSession s = i.next();
             s.clearMessagesInProgress();
         }
+        
+        for (ActiveMQConnectionConsumer connectionConsumer : this.connectionConsumers) {
+            connectionConsumer.clearMessagesInProgress();    
+        }
+        
         for (Iterator<TransportListener> iter = transportListeners.iterator(); iter.hasNext();) {
             TransportListener listener = iter.next();
             listener.transportInterupted();
@@ -2248,41 +2278,66 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
     }
 
     protected boolean isDuplicate(ActiveMQDispatcher dispatcher, Message message) {
-        return connectionAudit.isDuplicate(dispatcher, message);
+        return checkForDuplicates && connectionAudit.isDuplicate(dispatcher, message);
     }
 
     protected void rollbackDuplicate(ActiveMQDispatcher dispatcher, Message message) {
         connectionAudit.rollbackDuplicate(dispatcher, message);
     }
 
-    public IOException getFirstFailureError() {
-        return firstFailureError;
-    }
-
-
-    protected boolean willBlockWaitingForTransportInterruptionProcessing() throws InterruptedException {
-        boolean willBlock = false;
-        CountDownLatch latch = transportInterruptionProcessingComplete;
-        if (latch != null) {
-            willBlock = !closed.get() && !transportFailed.get() && latch.getCount() > 0;
-        }
-        return willBlock;
-    }
-
-    protected void waitForTransportInterruptionProcessing() throws InterruptedException {
-        CountDownLatch latch = transportInterruptionProcessingComplete;
-        if (latch != null) {
-            while (!closed.get() && !transportFailed.get() && !latch.await(5, TimeUnit.SECONDS)) {
-                LOG.warn("dispatch paused, waiting for outstanding dispatch interruption processing (" + latch.getCount() + ") to complete..");
+	public IOException getFirstFailureError() {
+		return firstFailureError;
+	}
+	
+	protected void waitForTransportInterruptionProcessingToComplete() throws InterruptedException {
+	    CountDownLatch cdl = this.transportInterruptionProcessingComplete;
+	    if (cdl != null) {
+            if (!closed.get() && !transportFailed.get() && cdl.getCount()>0) {
+                LOG.warn("dispatch paused, waiting for outstanding dispatch interruption processing (" + cdl.getCount() + ") to complete..");
+                cdl.await(10, TimeUnit.SECONDS);
             }
-            transportInterruptionProcessingComplete = null;
+            signalInterruptionProcessingComplete();
+        }
+    }
+	
+	protected void transportInterruptionProcessingComplete() {
+	    CountDownLatch cdl = this.transportInterruptionProcessingComplete;
+	    if (cdl != null) {
+	        cdl.countDown();
+	        try {
+	            signalInterruptionProcessingComplete();
+	        } catch (InterruptedException ignored) {}
+	    }
+	}
+
+    private void signalInterruptionProcessingComplete() throws InterruptedException {
+        CountDownLatch cdl = this.transportInterruptionProcessingComplete;
+        if (cdl.getCount()==0) {
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("transportInterruptionProcessingComplete for: " + this.getConnectionInfo().getConnectionId());
+            }
+            this.transportInterruptionProcessingComplete = null;
+
+            FailoverTransport failoverTransport = transport.narrow(FailoverTransport.class);
+            if (failoverTransport != null) {
+                failoverTransport.connectionInterruptProcessingComplete(this.getConnectionInfo().getConnectionId());
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("notified failover transport (" + failoverTransport
+                            + ") of interruption completion for: " + this.getConnectionInfo().getConnectionId());
+                }
+            }
+
         }
     }
 
-    protected synchronized void transportInterruptionProcessingComplete() {
-        CountDownLatch latch = transportInterruptionProcessingComplete;
-        if (latch != null) {
-            latch.countDown();
+    private void signalInterruptionProcessingNeeded() {
+        FailoverTransport failoverTransport = transport.narrow(FailoverTransport.class);
+        if (failoverTransport != null) {
+            failoverTransport.getStateTracker().transportInterrupted(this.getConnectionInfo().getConnectionId());
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("notified failover transport (" + failoverTransport
+                        + ") of pending interruption processing for: " + this.getConnectionInfo().getConnectionId());
+            }
         }
     }
 
@@ -2298,4 +2353,26 @@ public class ActiveMQConnection implements Connection, TopicConnection, QueueCon
     public long getConsumerFailoverRedeliveryWaitPeriod() {
         return consumerFailoverRedeliveryWaitPeriod;
     }
+    
+    protected Scheduler getScheduler() {
+        return this.scheduler;
+    }
+    
+    protected ThreadPoolExecutor getExecutor() {
+        return this.executor;
+    }
+
+    /**
+     * @return the checkForDuplicates
+     */
+    public boolean isCheckForDuplicates() {
+        return this.checkForDuplicates;
+    }
+
+    /**
+     * @param checkForDuplicates the checkForDuplicates to set
+     */
+    public void setCheckForDuplicates(boolean checkForDuplicates) {
+        this.checkForDuplicates = checkForDuplicates;
+    }
 }
