@@ -30,6 +30,7 @@ import java.util.Set;
 import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.LinkedBlockingQueue;
+import java.util.concurrent.SynchronousQueue;
 import java.util.concurrent.ThreadFactory;
 import java.util.concurrent.ThreadPoolExecutor;
 import java.util.concurrent.TimeUnit;
@@ -113,6 +114,8 @@ public class BrokerService implements Service {
     private boolean enableStatistics = true;
     private boolean persistent = true;
     private boolean populateJMSXUserID;
+    private boolean useAuthenticatedPrincipalForJMSXUserID;
+
     private boolean useShutdownHook = true;
     private boolean useLoggingForShutdownErrors;
     private boolean shutdownOnMasterFailure;
@@ -152,7 +155,7 @@ public class BrokerService implements Service {
     private boolean deleteAllMessagesOnStartup;
     private boolean advisorySupport = true;
     private URI vmConnectorURI;
-    private URI defaultSocketURI;
+    private String defaultSocketURIString;
     private PolicyMap destinationPolicy;
     private final AtomicBoolean started = new AtomicBoolean(false);
     private final AtomicBoolean stopped = new AtomicBoolean(false);
@@ -192,8 +195,8 @@ public class BrokerService implements Service {
     private boolean slave = true;
     private int schedulePeriodForDestinationPurge=5000;
     private BrokerContext brokerContext;
-    
-    
+    private boolean networkConnectorStartAsync = false;
+
 	static {
         String localHostName = "localhost";
         try {
@@ -1315,24 +1318,24 @@ public class BrokerService implements Service {
         this.vmConnectorURI = vmConnectorURI;
     }
     
-    public URI getDefaultSocketURI() {
+    public String getDefaultSocketURIString() {
        
             if (started.get()) {
-                if (this.defaultSocketURI==null) {
+                if (this.defaultSocketURIString ==null) {
                     for (TransportConnector tc:this.transportConnectors) {
-                        URI result = null;
+                        String result = null;
                         try {
-                            result = tc.getConnectUri();
+                            result = tc.getPublishableConnectString();
                         } catch (Exception e) {
                           LOG.warn("Failed to get the ConnectURI for "+tc,e);
                         }
                         if (result != null) {
-                            this.defaultSocketURI=result;
+                            this.defaultSocketURIString =result;
                             break;
                         }
                     }
                 }
-                return this.defaultSocketURI;
+                return this.defaultSocketURIString;
             }
        return null;
     }
@@ -1702,6 +1705,14 @@ public class BrokerService implements Service {
                 + "NetworkConnectorName=" + JMXSupport.encodeObjectNamePart(connector.getName()));
     }
 
+
+    public ObjectName createDuplexNetworkConnectorObjectName(String transport)
+            throws MalformedObjectNameException {
+        return new ObjectName(getManagementContext().getJmxDomainName() + ":" + "BrokerName="
+                + JMXSupport.encodeObjectNamePart(getBrokerName()) + "," + "Type=NetworkConnector,"
+                + "NetworkConnectorName=duplex" + JMXSupport.encodeObjectNamePart(transport));
+    }
+
     protected void unregisterNetworkConnectorMBean(NetworkConnector connector) {
         if (isUseJmx()) {
             try {
@@ -1874,7 +1885,9 @@ public class BrokerService implements Service {
         broker = new CompositeDestinationBroker(broker);
         broker = new TransactionBroker(broker, getPersistenceAdapter().createTransactionStore());
         if (isPopulateJMSXUserID()) {
-            broker = new UserIDBroker(broker);
+            UserIDBroker userIDBroker = new UserIDBroker(broker);
+            userIDBroker.setUseAuthenticatePrincipal(isUseAuthenticatedPrincipalForJMSXUserID());
+            broker = userIDBroker;
         }
         if (isMonitorConnectionSplits()) {
             broker = new ConnectionSplitBroker(broker);
@@ -2063,16 +2076,50 @@ public class BrokerService implements Service {
                 waitForSlave();
             }
             if (!stopped.get()) {
+                ThreadPoolExecutor networkConnectorStartExecutor = null;
+                if (isNetworkConnectorStartAsync()) {
+                    // spin up as many threads as needed
+                    networkConnectorStartExecutor = new ThreadPoolExecutor(0, Integer.MAX_VALUE,
+                            10, TimeUnit.SECONDS, new SynchronousQueue<Runnable>(),
+                            new ThreadFactory() {
+                                int count=0;
+                                public Thread newThread(Runnable runnable) {
+                                    Thread thread = new Thread(runnable, "NetworkConnector Start Thread-" +(count++));
+                                    thread.setDaemon(true);
+                                    return thread;
+                                }
+                            });
+                }
+
                 for (Iterator<NetworkConnector> iter = getNetworkConnectors().iterator(); iter.hasNext();) {
-                    NetworkConnector connector = iter.next();
+                    final NetworkConnector connector = iter.next();
                     connector.setLocalUri(uri);
                     connector.setBrokerName(getBrokerName());
                     connector.setDurableDestinations(durableDestinations);
-                    if (getDefaultSocketURI() != null) {
-                        connector.setBrokerURL(getDefaultSocketURI().toString());
+                    if (getDefaultSocketURIString() != null) {
+                        connector.setBrokerURL(getDefaultSocketURIString());
+                    }
+                    if (networkConnectorStartExecutor != null) {
+                        networkConnectorStartExecutor.execute(new Runnable() {
+                            public void run() {
+                                try {
+                                    LOG.info("Async start of " + connector);
+                                    connector.start();
+                                } catch(Exception e) {
+                                    LOG.error("Async start of network connector: " + connector + " failed", e);
+                                }
+                            }
+                        });
+                    } else {
+                        connector.start();
                     }
-                    connector.start();
                 }
+                if (networkConnectorStartExecutor != null) {
+                    // executor done when enqueued tasks are complete
+                    networkConnectorStartExecutor.shutdown();
+                    networkConnectorStartExecutor = null;
+                }
+
                 for (Iterator<ProxyConnector> iter = getProxyConnectors().iterator(); iter.hasNext();) {
                     ProxyConnector connector = iter.next();
                     connector.start();
@@ -2330,4 +2377,20 @@ public class BrokerService implements Service {
     public void setBrokerId(String brokerId) {
         this.brokerId = new BrokerId(brokerId);
     }
+
+    public boolean isUseAuthenticatedPrincipalForJMSXUserID() {
+        return useAuthenticatedPrincipalForJMSXUserID;
+    }
+
+    public void setUseAuthenticatedPrincipalForJMSXUserID(boolean useAuthenticatedPrincipalForJMSXUserID) {
+        this.useAuthenticatedPrincipalForJMSXUserID = useAuthenticatedPrincipalForJMSXUserID;
+    }
+
+    public boolean isNetworkConnectorStartAsync() {
+        return networkConnectorStartAsync;
+    }
+
+    public void setNetworkConnectorStartAsync(boolean networkConnectorStartAsync) {
+        this.networkConnectorStartAsync = networkConnectorStartAsync;
+    }
 }
