@@ -25,6 +25,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Properties;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
@@ -32,11 +33,13 @@ import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
 
+import javax.management.ObjectName;
 import javax.transaction.xa.XAResource;
 
 import org.apache.activemq.broker.ft.MasterBroker;
 import org.apache.activemq.broker.region.ConnectionStatistics;
 import org.apache.activemq.broker.region.RegionBroker;
+import org.apache.activemq.command.BrokerId;
 import org.apache.activemq.command.BrokerInfo;
 import org.apache.activemq.command.Command;
 import org.apache.activemq.command.CommandTypes;
@@ -70,9 +73,7 @@ import org.apache.activemq.command.ShutdownInfo;
 import org.apache.activemq.command.TransactionId;
 import org.apache.activemq.command.TransactionInfo;
 import org.apache.activemq.command.WireFormatInfo;
-import org.apache.activemq.network.DemandForwardingBridge;
-import org.apache.activemq.network.NetworkBridgeConfiguration;
-import org.apache.activemq.network.NetworkBridgeFactory;
+import org.apache.activemq.network.*;
 import org.apache.activemq.security.MessageAuthorizationPolicy;
 import org.apache.activemq.state.CommandVisitor;
 import org.apache.activemq.state.ConnectionState;
@@ -80,7 +81,6 @@ import org.apache.activemq.state.ConsumerState;
 import org.apache.activemq.state.ProducerState;
 import org.apache.activemq.state.SessionState;
 import org.apache.activemq.state.TransactionState;
-import org.apache.activemq.thread.DefaultThreadPools;
 import org.apache.activemq.thread.Task;
 import org.apache.activemq.thread.TaskRunner;
 import org.apache.activemq.thread.TaskRunnerFactory;
@@ -88,6 +88,7 @@ import org.apache.activemq.transaction.Transaction;
 import org.apache.activemq.transport.DefaultTransportListener;
 import org.apache.activemq.transport.ResponseCorrelator;
 import org.apache.activemq.transport.Transport;
+import org.apache.activemq.transport.TransportDisposedIOException;
 import org.apache.activemq.transport.TransportFactory;
 import org.apache.activemq.util.IntrospectionSupport;
 import org.apache.activemq.util.MarshallingSupport;
@@ -96,7 +97,7 @@ import org.apache.activemq.util.URISupport;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
-import static org.apache.activemq.thread.DefaultThreadPools.*;
+import static org.apache.activemq.thread.DefaultThreadPools.getDefaultTaskRunnerFactory;
 /**
  * @version $Revision: 1.8 $
  */
@@ -124,7 +125,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     // Used to do async dispatch.. this should perhaps be pushed down into the
     // transport layer..
     private boolean inServiceException;
-    private ConnectionStatistics statistics = new ConnectionStatistics();
+    private final ConnectionStatistics statistics = new ConnectionStatistics();
     private boolean manageable;
     private boolean slow;
     private boolean markedCandidate;
@@ -136,19 +137,20 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     private boolean pendingStop;
     private long timeStamp;
     private final AtomicBoolean stopping = new AtomicBoolean(false);
-    private CountDownLatch stopped = new CountDownLatch(1);
+    private final CountDownLatch stopped = new CountDownLatch(1);
     private final AtomicBoolean asyncException = new AtomicBoolean(false);
     private final Map<ProducerId, ProducerBrokerExchange> producerExchanges = new HashMap<ProducerId, ProducerBrokerExchange>();
     private final Map<ConsumerId, ConsumerBrokerExchange> consumerExchanges = new HashMap<ConsumerId, ConsumerBrokerExchange>();
-    private CountDownLatch dispatchStoppedLatch = new CountDownLatch(1);
+    private final CountDownLatch dispatchStoppedLatch = new CountDownLatch(1);
     private ConnectionContext context;
     private boolean networkConnection;
     private boolean faultTolerantConnection;
-    private AtomicInteger protocolVersion = new AtomicInteger(CommandTypes.PROTOCOL_VERSION);
+    private final AtomicInteger protocolVersion = new AtomicInteger(CommandTypes.PROTOCOL_VERSION);
     private DemandForwardingBridge duplexBridge;
     private final TaskRunnerFactory taskRunnerFactory;
     private TransportConnectionStateRegister connectionStateRegister = new SingleTransportConnectionStateRegister();
     private final ReentrantReadWriteLock serviceLock = new ReentrantReadWriteLock();
+    private String duplexNetworkConnectorId;
 
     /**
      * @param connector
@@ -171,6 +173,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
         this.taskRunnerFactory = taskRunnerFactory;
         this.transport = transport;
         this.transport.setTransportListener(new DefaultTransportListener() {
+            @Override
             public void onCommand(Object o) {
                 serviceLock.readLock().lock();
                 try {
@@ -187,6 +190,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
                 }
             }
 
+            @Override
             public void onException(IOException exception) {
                 serviceLock.readLock().lock();
                 try {
@@ -229,6 +233,8 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
             transportException.set(e);
             if (TRANSPORTLOG.isDebugEnabled()) {
                 TRANSPORTLOG.debug("Transport failed: " + e, e);
+            } else if (TRANSPORTLOG.isInfoEnabled()) {
+                TRANSPORTLOG.info("Transport failed: " + e);
             }
             stopAsync();
         }
@@ -244,6 +250,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     public void serviceExceptionAsync(final IOException e) {
         if (asyncException.compareAndSet(false, true)) {
             new Thread("Async Exception Handler") {
+                @Override
                 public void run() {
                     serviceException(e);
                 }
@@ -450,7 +457,9 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     public Response processMessage(Message messageSend) throws Exception {
         ProducerId producerId = messageSend.getProducerId();
         ProducerBrokerExchange producerExchange = getProducerBrokerExchange(producerId);
-        broker.send(producerExchange, messageSend);
+        if (producerExchange.canDispatch(messageSend)) {
+            broker.send(producerExchange, messageSend);
+        }
         return null;
     }
 
@@ -575,7 +584,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
         ConnectionId connectionId = info.getSessionId().getParentId();
         TransportConnectionState cs = lookupConnectionState(connectionId);
         // Avoid replaying dup commands
-        if (!cs.getSessionIds().contains(info.getSessionId())) {
+        if (cs != null && !cs.getSessionIds().contains(info.getSessionId())) {
             broker.addSession(cs.getContext(), info);
             try {
                 cs.addSession(info);
@@ -661,6 +670,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
         }
         registerConnectionState(info.getConnectionId(), state);
         LOG.debug("Setting up new connection id: " + info.getConnectionId() + ", address: " + getRemoteAddress());
+        this.faultTolerantConnection=info.isFaultTolerant();
         // Setup the context.
         String clientId = info.getClientId();
         context = new ConnectionContext();
@@ -676,19 +686,27 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
         context.setTransactions(new ConcurrentHashMap<TransactionId, Transaction>());
         context.setUserName(info.getUserName());
         context.setWireFormatInfo(wireFormatInfo);
+        context.setReconnect(info.isFailoverReconnect());
         this.manageable = info.isManageable();
         state.setContext(context);
         state.setConnection(this);
+       
         try {
             broker.addConnection(context, info);
         } catch (Exception e) {
-            brokerConnectionStates.remove(info);
-            LOG.warn("Failed to add Connection", e);
+            synchronized (brokerConnectionStates) {
+                brokerConnectionStates.remove(info.getConnectionId());
+            }
+            unregisterConnectionState(info.getConnectionId());
+            LOG.warn("Failed to add Connection " + info.getConnectionId() + ", reason: " +  e.toString());
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Exception detail:", e);
+            }
             throw e;
         }
-        if (info.isManageable() && broker.isFaultTolerantConfiguration()) {
+        if (info.isManageable()) {
             // send ConnectionCommand
-            ConnectionControl command = new ConnectionControl();
+            ConnectionControl command = this.connector.getConnectionControl();
             command.setFaultTolerant(broker.isFaultTolerantConfiguration());
             dispatchAsync(command);
         }
@@ -726,7 +744,10 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
             try {
                 broker.removeConnection(cs.getContext(), cs.getInfo(), null);
             } catch (Throwable e) {
-                SERVICELOG.warn("Failed to remove connection " + cs.getInfo(), e);
+                SERVICELOG.warn("Failed to remove connection " + cs.getInfo() + ", reason: " + e.toString());
+                if (LOG.isDebugEnabled()) {
+                    SERVICELOG.debug("Exception detail:", e);
+                }
             }
             TransportConnectionState state = unregisterConnectionState(id);
             if (state != null) {
@@ -874,7 +895,14 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
                 }
                 transport.start();
                 active = true;
-                dispatchAsync(connector.getBrokerInfo());
+                BrokerInfo info = connector.getBrokerInfo().copy();
+                if (connector.isUpdateClusterClients()) {
+                    info.setPeerBrokerInfos(this.broker.getPeerBrokerInfos());
+                } else {
+                    info.setPeerBrokerInfos(null);
+                }
+                dispatchAsync(info);
+                
                 connector.onStarted(this);
             }
         } catch (Exception e) {
@@ -931,7 +959,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
                             serviceLock.writeLock().unlock();
                         }
                     }
-                });
+                }, "StopAsync:" + transport.getRemoteAddress());
             } catch (Throwable t) {
                 LOG.warn("cannot create async transport stopper thread.. not waiting for stop to complete, reason:", t);
                 stopped.countDown();
@@ -1000,9 +1028,6 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
                     ignore.printStackTrace();
                 }
             }
-            if (brokerInfo != null) {
-                broker.removeBroker(this, brokerInfo);
-            }
         }
         LOG.debug("Connection Stopped: " + getRemoteAddress());
     }
@@ -1127,6 +1152,10 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     public synchronized boolean isNetworkConnection() {
         return networkConnection;
     }
+    
+    public boolean isFaultTolerantConnection() {
+       return this.faultTolerantConnection;
+    }
 
     protected synchronized void setStarting(boolean starting) {
         this.starting = starting;
@@ -1168,23 +1197,54 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
                 NetworkBridgeConfiguration config = new NetworkBridgeConfiguration();
                 IntrospectionSupport.setProperties(config, props, "");
                 config.setBrokerName(broker.getBrokerName());
+
+                // check for existing duplex connection hanging about
+
+                // We first look if existing network connection already exists for the same broker Id and network connector name
+                // It's possible in case of brief network fault to have this transport connector side of the connection always active
+                // and the duplex network connector side wanting to open a new one
+                // In this case, the old connection must be broken
+                String duplexNetworkConnectorId = config.getName() + "@" + info.getBrokerId(); 
+                CopyOnWriteArrayList<TransportConnection> connections = this.connector.getConnections();
+                synchronized (connections) {
+                    for (Iterator<TransportConnection> iter = connections.iterator(); iter.hasNext();) {
+                        TransportConnection c = iter.next();
+                        if ((c != this) && (duplexNetworkConnectorId.equals(c.getDuplexNetworkConnectorId()))) {
+                            LOG.warn("Stopping an existing active duplex connection [" + c + "] for network connector (" + duplexNetworkConnectorId + ").");
+                            c.stopAsync();
+                            // better to wait for a bit rather than get connection id already in use and failure to start new bridge
+                            c.getStopped().await(1, TimeUnit.SECONDS);
+                        }
+                    }
+                    setDuplexNetworkConnectorId(duplexNetworkConnectorId);
+                }
                 URI uri = broker.getVmConnectorURI();
-                HashMap<String, String> map = new HashMap<String, String>(URISupport.parseParamters(uri));
+                HashMap<String, String> map = new HashMap<String, String>(URISupport.parseParameters(uri));
                 map.put("network", "true");
                 map.put("async", "false");
                 uri = URISupport.createURIWithQuery(uri, URISupport.createQueryString(map));
                 Transport localTransport = TransportFactory.connect(uri);
                 Transport remoteBridgeTransport = new ResponseCorrelator(transport);
-                duplexBridge = NetworkBridgeFactory.createBridge(config, localTransport, remoteBridgeTransport);
+                String duplexName = localTransport.toString();
+                if (duplexName.contains("#")) {
+                    duplexName = duplexName.substring(duplexName.lastIndexOf("#"));
+                }
+                MBeanNetworkListener listener = new MBeanNetworkListener(broker.getBrokerService(), broker.getBrokerService().createDuplexNetworkConnectorObjectName(duplexName));
+                listener.setCreatedByDuplex(true);
+                duplexBridge = NetworkBridgeFactory.createBridge(config, localTransport, remoteBridgeTransport, listener);
                 duplexBridge.setBrokerService(broker.getBrokerService());
                 // now turn duplex off this side
                 info.setDuplexConnection(false);
                 duplexBridge.setCreatedByDuplex(true);
                 duplexBridge.duplexStart(this, brokerInfo, info);
-                LOG.info("Created Duplex Bridge back to " + info.getBrokerName());
+                LOG.info("Started responder end of duplex bridge " + duplexNetworkConnectorId);
+                return null;
+            } catch (TransportDisposedIOException e) {
+                LOG.warn("Duplex bridge " + duplexNetworkConnectorId + " was stopped before it was correctly started.");
                 return null;
             } catch (Exception e) {
-                LOG.error("Creating duplex network bridge", e);
+                LOG.error("Failed to create responder end of duplex network bridge " + duplexNetworkConnectorId , e);
+                return null;
             }
         }
         // We only expect to get one broker info command per connection
@@ -1192,7 +1252,6 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
             LOG.warn("Unexpected extra broker info command received: " + info);
         }
         this.brokerInfo = info;
-        broker.addBroker(this, info);
         networkConnection = true;
         List<TransportConnectionState> connectionStates = listConnectionStates();
         for (TransportConnectionState cs : connectionStates) {
@@ -1229,14 +1288,24 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
         }
         return null;
     }
+        
+    public void updateClient(ConnectionControl control) {
+        if (isActive() && isBlocked() == false && isFaultTolerantConnection() && this.wireFormatInfo != null
+                && this.wireFormatInfo.getVersion() >= 6) {
+            dispatchAsync(control);
+        }
+    }
 
-    private ProducerBrokerExchange getProducerBrokerExchange(ProducerId id) {
+    private ProducerBrokerExchange getProducerBrokerExchange(ProducerId id) throws IOException {
         ProducerBrokerExchange result = producerExchanges.get(id);
         if (result == null) {
             synchronized (producerExchanges) {
                 result = new ProducerBrokerExchange();
-                TransportConnectionState state = lookupConnectionState(id);
+                TransportConnectionState state = lookupConnectionState(id);              
                 context = state.getContext();
+                if (context.isReconnect()) {
+                    result.setLastStoredSequenceId(broker.getBrokerService().getPersistenceAdapter().getLastProducerSequenceId(id));
+                }
                 result.setConnectionContext(context);
                 SessionState ss = state.getSessionState(id.getParentId());
                 if (ss != null) {
@@ -1321,6 +1390,8 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     }
 
     public Response processConsumerControl(ConsumerControl control) throws Exception {
+        ConsumerBrokerExchange consumerExchange = getConsumerBrokerExchange(control.getConsumerId());
+        broker.processConsumerControl(consumerExchange, control);
         return null;
     }
 
@@ -1364,4 +1435,16 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     protected synchronized TransportConnectionState lookupConnectionState(ConnectionId connectionId) {
         return connectionStateRegister.lookupConnectionState(connectionId);
     }
+
+    protected synchronized void setDuplexNetworkConnectorId(String duplexNetworkConnectorId) {
+        this.duplexNetworkConnectorId = duplexNetworkConnectorId;
+    }
+
+    protected synchronized String getDuplexNetworkConnectorId() {
+        return this.duplexNetworkConnectorId;
+    }
+    
+    protected CountDownLatch getStopped() {
+        return stopped;
+    }
 }
