@@ -40,11 +40,12 @@ public abstract class AbstractPendingMessageCursor implements PendingMessageCurs
     protected int memoryUsageHighWaterMark = 70;
     protected int maxBatchSize = BaseDestination.MAX_PAGE_SIZE;
     protected SystemUsage systemUsage;
-    protected int maxProducersToAudit=1024;
-    protected int maxAuditDepth=1000;
+    protected int maxProducersToAudit = BaseDestination.MAX_PRODUCERS_TO_AUDIT;
+    protected int maxAuditDepth = BaseDestination.MAX_AUDIT_DEPTH;
     protected boolean enableAudit=true;
     protected ActiveMQMessageAudit audit;
     protected boolean useCache=true;
+    private boolean cacheEnabled=true;
     private boolean started=false;
     protected MessageReference last = null;
     protected final boolean prioritizedMessages;
@@ -327,4 +328,12 @@ public abstract class AbstractPendingMessageCursor implements PendingMessageCurs
         return result;
 
     }
+
+    public synchronized boolean isCacheEnabled() {
+        return cacheEnabled;
+    }
+
+    public synchronized void setCacheEnabled(boolean val) {
+        cacheEnabled = val;
+    }
 }
