@@ -38,7 +38,7 @@ public class Statements {
     private String tablePrefix = "";
     private String addMessageStatement;
     private String updateMessageStatement;
-    private String removeMessageStatment;
+    private String removeMessageStatement;
     private String findMessageSequenceIdStatement;
     private String findMessageStatement;
     private String findMessageByIdStatement;
@@ -48,10 +48,11 @@ public class Statements {
     private String createDurableSubStatement;
     private String findDurableSubStatement;
     private String findAllDurableSubsStatement;
-    private String updateLastAckOfDurableSubStatement;
+    private String updateLastPriorityAckRowOfDurableSubStatement;
     private String deleteSubscriptionStatement;
     private String findAllDurableSubMessagesStatement;
     private String findDurableSubMessagesStatement;
+    private String findDurableSubMessagesByPriorityStatement;
     private String findAllDestinationsStatement;
     private String removeAllMessagesStatement;
     private String removeAllSubscriptionsStatement;
@@ -65,8 +66,17 @@ public class Statements {
     private String lastAckedDurableSubscriberMessageStatement;
     private String destinationMessageCountStatement;
     private String findNextMessagesStatement;
+    private String findNextMessagesByPriorityStatement;
     private boolean useLockCreateWhereClause;
     private String findAllMessageIdsStatement;
+    private String lastProducerSequenceIdStatement;
+    private String selectDurablePriorityAckStatement;
+
+    private String insertDurablePriorityAckStatement;
+    private String updateDurableLastAckStatement;
+    private String deleteOldMessagesStatementWithPriority;
+    private String durableSubscriberMessageCountStatementWithPriority;
+    private String dropAckPKAlterStatementEnd;
 
     public String[] getCreateSchemaStatements() {
         if (createSchemaStatements == null) {
@@ -88,11 +98,27 @@ public class Statements {
                     + "( ID " + longDataType + " NOT NULL, TIME " + longDataType 
                     + ", BROKER_NAME " + stringIdDataType + ", PRIMARY KEY (ID) )",
                 "INSERT INTO " + getFullLockTableName() + "(ID) VALUES (1)", 
+                "ALTER TABLE " + getFullMessageTableName() + " ADD PRIORITY " + sequenceDataType,
+                "CREATE INDEX " + getFullMessageTableName() + "_PIDX ON " + getFullMessageTableName() + " (PRIORITY)",
+                "ALTER TABLE " + getFullAckTableName() + " ADD PRIORITY " + sequenceDataType  + " DEFAULT 5 NOT NULL",
+                "ALTER TABLE " + getFullAckTableName() + " " + getDropAckPKAlterStatementEnd(),
+                "ALTER TABLE " + getFullAckTableName() + " ADD PRIMARY KEY (CONTAINER, CLIENT_ID, SUB_NAME, PRIORITY)",
             };
         }
         return createSchemaStatements;
     }
 
+    public String getDropAckPKAlterStatementEnd() {
+        if (dropAckPKAlterStatementEnd == null) {
+            dropAckPKAlterStatementEnd = "DROP PRIMARY KEY";
+        }
+        return dropAckPKAlterStatementEnd;
+    }
+
+    public void setDropAckPKAlterStatementEnd(String dropAckPKAlterStatementEnd) {
+        this.dropAckPKAlterStatementEnd = dropAckPKAlterStatementEnd;
+    }
+
     public String[] getDropSchemaStatements() {
         if (dropSchemaStatements == null) {
             dropSchemaStatements = new String[] {"DROP TABLE " + getFullAckTableName() + "",
@@ -106,7 +132,7 @@ public class Statements {
         if (addMessageStatement == null) {
             addMessageStatement = "INSERT INTO "
                                   + getFullMessageTableName()
-                                  + "(ID, MSGID_PROD, MSGID_SEQ, CONTAINER, EXPIRATION, MSG) VALUES (?, ?, ?, ?, ?, ?)";
+                                  + "(ID, MSGID_PROD, MSGID_SEQ, CONTAINER, EXPIRATION, PRIORITY, MSG) VALUES (?, ?, ?, ?, ?, ?, ?)";
         }
         return addMessageStatement;
     }
@@ -118,17 +144,17 @@ public class Statements {
         return updateMessageStatement;
     }
 
-    public String getRemoveMessageStatment() {
-        if (removeMessageStatment == null) {
-            removeMessageStatment = "DELETE FROM " + getFullMessageTableName() + " WHERE ID=?";
+    public String getRemoveMessageStatement() {
+        if (removeMessageStatement == null) {
+            removeMessageStatement = "DELETE FROM " + getFullMessageTableName() + " WHERE ID=?";
         }
-        return removeMessageStatment;
+        return removeMessageStatement;
     }
 
     public String getFindMessageSequenceIdStatement() {
         if (findMessageSequenceIdStatement == null) {
-            findMessageSequenceIdStatement = "SELECT ID FROM " + getFullMessageTableName()
-                                             + " WHERE MSGID_PROD=? AND MSGID_SEQ=?";
+            findMessageSequenceIdStatement = "SELECT ID, PRIORITY FROM " + getFullMessageTableName()
+                                             + " WHERE MSGID_PROD=? AND MSGID_SEQ=? AND CONTAINER=?";
         }
         return findMessageSequenceIdStatement;
     }
@@ -141,10 +167,10 @@ public class Statements {
     }
 
     public String getFindMessageByIdStatement() {
-        if (findMessageStatement == null) {
-            findMessageStatement = "SELECT MSG FROM " + getFullMessageTableName() + " WHERE ID=?";
+        if (findMessageByIdStatement == null) {
+        	findMessageByIdStatement = "SELECT MSG FROM " + getFullMessageTableName() + " WHERE ID=?";
         }
-        return findMessageStatement;
+        return findMessageByIdStatement;
     }
     
     public String getFindAllMessagesStatement() {
@@ -172,6 +198,15 @@ public class Statements {
         return findLastSequenceIdInMsgsStatement;
     }
 
+    public String getLastProducerSequenceIdStatement() {
+        if (lastProducerSequenceIdStatement == null) {
+            lastProducerSequenceIdStatement = "SELECT MAX(MSGID_SEQ) FROM " + getFullMessageTableName()
+                                            + " WHERE MSGID_PROD=?";
+        }
+        return lastProducerSequenceIdStatement;
+    }
+
+
     public String getFindLastSequenceIdInAcksStatement() {
         if (findLastSequenceIdInAcksStatement == null) {
             findLastSequenceIdInAcksStatement = "SELECT MAX(LAST_ACKED_ID) FROM " + getFullAckTableName();
@@ -183,8 +218,8 @@ public class Statements {
         if (createDurableSubStatement == null) {
             createDurableSubStatement = "INSERT INTO "
                                         + getFullAckTableName()
-                                        + "(CONTAINER, CLIENT_ID, SUB_NAME, SELECTOR, LAST_ACKED_ID, SUB_DEST) "
-                                        + "VALUES (?, ?, ?, ?, ?, ?)";
+                                        + "(CONTAINER, CLIENT_ID, SUB_NAME, SELECTOR, LAST_ACKED_ID, SUB_DEST, PRIORITY) "
+                                        + "VALUES (?, ?, ?, ?, ?, ?, ?)";
         }
         return createDurableSubStatement;
     }
@@ -200,17 +235,17 @@ public class Statements {
     public String getFindAllDurableSubsStatement() {
         if (findAllDurableSubsStatement == null) {
             findAllDurableSubsStatement = "SELECT SELECTOR, SUB_NAME, CLIENT_ID, SUB_DEST" + " FROM "
-                                          + getFullAckTableName() + " WHERE CONTAINER=?";
+                                          + getFullAckTableName() + " WHERE CONTAINER=? AND PRIORITY=0";
         }
         return findAllDurableSubsStatement;
     }
 
-    public String getUpdateLastAckOfDurableSubStatement() {
-        if (updateLastAckOfDurableSubStatement == null) {
-            updateLastAckOfDurableSubStatement = "UPDATE " + getFullAckTableName() + " SET LAST_ACKED_ID=?"
-                                                 + " WHERE CONTAINER=? AND CLIENT_ID=? AND SUB_NAME=?";
+    public String getUpdateLastPriorityAckRowOfDurableSubStatement() {
+        if (updateLastPriorityAckRowOfDurableSubStatement == null) {
+            updateLastPriorityAckRowOfDurableSubStatement = "UPDATE " + getFullAckTableName() + " SET LAST_ACKED_ID=?"
+                                                 + " WHERE CONTAINER=? AND CLIENT_ID=? AND SUB_NAME=? AND PRIORITY=?";
         }
-        return updateLastAckOfDurableSubStatement;
+        return updateLastPriorityAckRowOfDurableSubStatement;
     }
 
     public String getDeleteSubscriptionStatement() {
@@ -227,7 +262,7 @@ public class Statements {
                                                  + " M, " + getFullAckTableName() + " D "
                                                  + " WHERE D.CONTAINER=? AND D.CLIENT_ID=? AND D.SUB_NAME=?"
                                                  + " AND M.CONTAINER=D.CONTAINER AND M.ID > D.LAST_ACKED_ID"
-                                                 + " ORDER BY M.ID";
+                                                 + " ORDER BY M.PRIORITY DESC, M.ID";
         }
         return findAllDurableSubMessagesStatement;
     }
@@ -237,11 +272,25 @@ public class Statements {
             findDurableSubMessagesStatement = "SELECT M.ID, M.MSG FROM " + getFullMessageTableName() + " M, "
                                               + getFullAckTableName() + " D "
                                               + " WHERE D.CONTAINER=? AND D.CLIENT_ID=? AND D.SUB_NAME=?"
-                                              + " AND M.CONTAINER=D.CONTAINER AND M.ID > ?"
+                                              + " AND M.CONTAINER=D.CONTAINER AND M.ID > D.LAST_ACKED_ID"
+                                              + " AND M.ID > ?"
                                               + " ORDER BY M.ID";
         }
         return findDurableSubMessagesStatement;
     }
+    
+    public String getFindDurableSubMessagesByPriorityStatement() {
+        if (findDurableSubMessagesByPriorityStatement == null) {
+            findDurableSubMessagesByPriorityStatement = "SELECT M.ID, M.MSG FROM " + getFullMessageTableName() + " M,"
+                                              + " " + getFullAckTableName() + " D"
+                                              + " WHERE D.CONTAINER=? AND D.CLIENT_ID=? AND D.SUB_NAME=?"
+                                              + " AND M.CONTAINER=D.CONTAINER"
+                                              + " AND M.PRIORITY=D.PRIORITY AND M.ID > D.LAST_ACKED_ID"
+                                              + " AND M.ID > ? AND M.PRIORITY = ?"
+                                              + " ORDER BY M.ID";
+        }
+        return findDurableSubMessagesByPriorityStatement;
+    }    
 
     public String findAllDurableSubMessagesStatement() {
         if (findAllDurableSubMessagesStatement == null) {
@@ -280,14 +329,35 @@ public class Statements {
                                                      + getFullAckTableName()
                                                      + " D "
                                                      + " WHERE D.CONTAINER=? AND D.CLIENT_ID=? AND D.SUB_NAME=?"
-                                                     + " AND M.CONTAINER=D.CONTAINER AND M.ID > D.LAST_ACKED_ID";
+                                                     + " AND M.CONTAINER=D.CONTAINER "
+                                                     + "     AND M.ID >"
+                                                     + "          ( SELECT LAST_ACKED_ID FROM " + getFullAckTableName()
+                                                     + "           WHERE CONTAINER=D.CONTAINER AND CLIENT_ID=D.CLIENT_ID"
+                                                     + "           AND SUB_NAME=D.SUB_NAME )";
+
         }
         return durableSubscriberMessageCountStatement;
     }
 
+    public String getDurableSubscriberMessageCountStatementWithPriority() {
+        if (durableSubscriberMessageCountStatementWithPriority == null) {
+            durableSubscriberMessageCountStatementWithPriority = "SELECT COUNT(*) FROM "
+                                                     + getFullMessageTableName()
+                                                     + " M, "
+                                                     + getFullAckTableName()
+                                                     + " D "
+                                                     + " WHERE D.CONTAINER=? AND D.CLIENT_ID=? AND D.SUB_NAME=?"
+                                                     + " AND M.CONTAINER=D.CONTAINER "
+                                                     + " AND M.PRIORITY=D.PRIORITY "
+                                                     + " AND M.ID > D.LAST_ACKED_ID";
+        }
+
+        return durableSubscriberMessageCountStatementWithPriority;
+    }
+
     public String getFindAllDestinationsStatement() {
         if (findAllDestinationsStatement == null) {
-            findAllDestinationsStatement = "SELECT DISTINCT CONTAINER FROM " + getFullMessageTableName();
+            findAllDestinationsStatement = "SELECT DISTINCT CONTAINER FROM " + getFullAckTableName();
         }
         return findAllDestinationsStatement;
     }
@@ -306,18 +376,37 @@ public class Statements {
         return removeAllSubscriptionsStatement;
     }
 
+    public String getDeleteOldMessagesStatementWithPriority() {
+        if (deleteOldMessagesStatementWithPriority == null) {
+            deleteOldMessagesStatementWithPriority = "DELETE FROM " + getFullMessageTableName()
+                                         + " WHERE ( EXPIRATION<>0 AND EXPIRATION<?)"
+                                         + " OR (ID <= "
+                                         + "     ( SELECT min(" + getFullAckTableName() + ".LAST_ACKED_ID)"
+                                         + "       FROM " + getFullAckTableName() + " WHERE "
+                                         +          getFullAckTableName() + ".CONTAINER="
+                                         +          getFullMessageTableName() + ".CONTAINER"
+                                         + "        AND " + getFullAckTableName() + ".PRIORITY=" + getFullMessageTableName() + ".PRIORITY )"
+                                         + "   )";
+        }
+        return deleteOldMessagesStatementWithPriority;
+    }
+
     public String getDeleteOldMessagesStatement() {
         if (deleteOldMessagesStatement == null) {
             deleteOldMessagesStatement = "DELETE FROM " + getFullMessageTableName()
-                                         + " WHERE ( EXPIRATION<>0 AND EXPIRATION<?) OR ID <= "
-                                         + "( SELECT min(" + getFullAckTableName() + ".LAST_ACKED_ID) "
-                                         + "FROM " + getFullAckTableName() + " WHERE "
-                                         + getFullAckTableName() + ".CONTAINER=" + getFullMessageTableName()
-                                         + ".CONTAINER)";
+                                         + " WHERE ( EXPIRATION<>0 AND EXPIRATION<?)"
+                                         + " OR (ID <= "
+                                         + "     ( SELECT min(" + getFullAckTableName() + ".LAST_ACKED_ID)"
+                                         + "       FROM " + getFullAckTableName() + " WHERE "
+                                         +          getFullAckTableName() + ".CONTAINER="
+                                         +          getFullMessageTableName() + ".CONTAINER )"
+                                         + "   )";
+
         }
         return deleteOldMessagesStatement;
     }
 
+
     public String getLockCreateStatement() {
         if (lockCreateStatement == null) {
             lockCreateStatement = "SELECT * FROM " + getFullLockTableName();
@@ -358,6 +447,19 @@ public class Statements {
         return findNextMessagesStatement;
     }
 
+    /**
+     * @return the findNextMessagesStatement
+     */
+    public String getFindNextMessagesByPriorityStatement() {
+        if (findNextMessagesByPriorityStatement == null) {
+            findNextMessagesByPriorityStatement = "SELECT ID, MSG FROM " + getFullMessageTableName()
+                                        + " WHERE CONTAINER=?"
+                                        + " AND ((ID > ? AND PRIORITY = ?) OR PRIORITY < ?)"
+                                        + " ORDER BY PRIORITY DESC, ID";
+        }
+        return findNextMessagesByPriorityStatement;
+    }    
+    
     /**
      * @return the lastAckedDurableSubscriberMessageStatement
      */
@@ -365,11 +467,39 @@ public class Statements {
         if (lastAckedDurableSubscriberMessageStatement == null) {
             lastAckedDurableSubscriberMessageStatement = "SELECT MAX(LAST_ACKED_ID) FROM "
                                                          + getFullAckTableName()
-                                                         + " WHERE CONTAINER=? AND CLIENT_ID=? AND SUB_NAME=?";
+                                                         + " WHERE CONTAINER=? AND CLIENT_ID=? AND SUB_NAME=?";                                                    
         }
         return lastAckedDurableSubscriberMessageStatement;
     }
 
+    public String getSelectDurablePriorityAckStatement() {
+        if (selectDurablePriorityAckStatement == null) {
+            selectDurablePriorityAckStatement = "SELECT LAST_ACKED_ID FROM " + getFullAckTableName()
+                                                    + " WHERE CONTAINER=? AND CLIENT_ID=? AND SUB_NAME=?"
+                                                    + " AND PRIORITY = ?";
+        }
+        return selectDurablePriorityAckStatement;
+    }
+
+    public String getInsertDurablePriorityAckStatement() {
+        if (insertDurablePriorityAckStatement == null) {
+            insertDurablePriorityAckStatement = "INSERT INTO "
+                                  + getFullAckTableName()
+                                  + "(CONTAINER, CLIENT_ID, SUB_NAME, PRIORITY)"
+                                  + " VALUES (?, ?, ?, ?)";            
+       }
+        return insertDurablePriorityAckStatement;
+    }
+
+
+    public String getUpdateDurableLastAckStatement() {
+        if (updateDurableLastAckStatement == null) {
+            updateDurableLastAckStatement  = "UPDATE " + getFullAckTableName()
+                    + " SET LAST_ACKED_ID = ? WHERE CONTAINER=? AND CLIENT_ID=? AND SUB_NAME=?";
+        }
+        return  updateDurableLastAckStatement;
+    }
+
     public String getFullMessageTableName() {
         return getTablePrefix() + getMessageTableName();
     }
@@ -528,6 +658,10 @@ public class Statements {
         this.deleteOldMessagesStatement = deleteOldMessagesStatment;
     }
 
+    public void setDeleteOldMessagesStatementWithPriority(String deleteOldMessagesStatmentWithPriority) {
+        this.deleteOldMessagesStatementWithPriority = deleteOldMessagesStatmentWithPriority;
+    }
+
     public void setDeleteSubscriptionStatement(String deleteSubscriptionStatment) {
         this.deleteSubscriptionStatement = deleteSubscriptionStatment;
     }
@@ -571,6 +705,10 @@ public class Statements {
     public void setFindMessageStatement(String findMessageStatment) {
         this.findMessageStatement = findMessageStatment;
     }
+    
+    public void setFindMessageByIdStatement(String findMessageByIdStatement) {
+        this.findMessageByIdStatement = findMessageByIdStatement;
+    }
 
     public void setRemoveAllMessagesStatement(String removeAllMessagesStatment) {
         this.removeAllMessagesStatement = removeAllMessagesStatment;
@@ -580,12 +718,12 @@ public class Statements {
         this.removeAllSubscriptionsStatement = removeAllSubscriptionsStatment;
     }
 
-    public void setRemoveMessageStatment(String removeMessageStatment) {
-        this.removeMessageStatment = removeMessageStatment;
+    public void setRemoveMessageStatment(String removeMessageStatement) {
+        this.removeMessageStatement = removeMessageStatement;
     }
 
-    public void setUpdateLastAckOfDurableSubStatement(String updateLastAckOfDurableSub) {
-        this.updateLastAckOfDurableSubStatement = updateLastAckOfDurableSub;
+    public void setUpdateLastPriorityAckRowOfDurableSubStatement(String updateLastPriorityAckRowOfDurableSubStatement) {
+        this.updateLastPriorityAckRowOfDurableSubStatement = updateLastPriorityAckRowOfDurableSubStatement;
     }
 
     public void setUpdateMessageStatement(String updateMessageStatment) {
@@ -630,6 +768,10 @@ public class Statements {
         this.durableSubscriberMessageCountStatement = durableSubscriberMessageCountStatement;
     }
 
+    public void setDurableSubscriberMessageCountStatementWithPriority(String durableSubscriberMessageCountStatementWithPriority) {
+        this.durableSubscriberMessageCountStatementWithPriority = durableSubscriberMessageCountStatementWithPriority;
+    }
+
     /**
      * @param findNextMessagesStatement the findNextMessagesStatement to set
      */
@@ -652,4 +794,20 @@ public class Statements {
         this.lastAckedDurableSubscriberMessageStatement = lastAckedDurableSubscriberMessageStatement;
     }
 
+
+    public void setLastProducerSequenceIdStatement(String lastProducerSequenceIdStatement) {
+        this.lastProducerSequenceIdStatement = lastProducerSequenceIdStatement;
+    }
+
+    public void setSelectDurablePriorityAckStatement(String selectDurablePriorityAckStatement) {
+        this.selectDurablePriorityAckStatement = selectDurablePriorityAckStatement;
+    }
+
+    public void setInsertDurablePriorityAckStatement(String insertDurablePriorityAckStatement) {
+        this.insertDurablePriorityAckStatement = insertDurablePriorityAckStatement;
+    }
+
+    public void setUpdateDurableLastAckStatement(String updateDurableLastAckStatement) {
+        this.updateDurableLastAckStatement = updateDurableLastAckStatement;
+    }    
 }
