@@ -487,15 +487,13 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                         StoredDestination sd = getStoredDestination(dest, tx);
                         Entry<Long, MessageKeys> entry = null;
                         int counter = 0;
-                        for (Iterator<Entry<Long, MessageKeys>> iterator = sd.orderIndex.iterator(tx); iterator
-                                .hasNext()
-                                && listener.hasSpace();) {
+                        for (Iterator<Entry<Long, MessageKeys>> iterator = sd.orderIndex.iterator(tx);
+                             listener.hasSpace() && iterator.hasNext(); ) {
                             entry = iterator.next();
                             Message msg = loadMessage(entry.getValue().location);
-                            //System.err.println("RECOVER " + msg.getMessageId().getProducerSequenceId());
                             listener.recoverMessage(msg);
                             counter++;
-                            if (counter >= maxReturned || listener.hasSpace() == false) {
+                            if (counter >= maxReturned) {
                                 break;
                             }
                         }
@@ -531,7 +529,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                 // operations... but for now we must
                 // externally synchronize...
                
-                indexLock.readLock().lock();
+                indexLock.writeLock().lock();
                 try {
                         pageFile.tx().execute(new Transaction.Closure<IOException>() {
                         public void execute(Transaction tx) throws IOException {
@@ -543,7 +541,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                         }
                     });
                 }finally {
-                    indexLock.readLock().unlock();
+                    indexLock.writeLock().unlock();
                 }
                 
             } finally {
@@ -718,57 +716,26 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
             }
         }
 
-        // an ack for an unmatched message is stored as a negative sequence id
-        // if sub has been getting unmatched acks, we need to reset
-        protected Long resetForSelectors(SubscriptionInfo info, Long position) {
-            if (info.getSelector() != null) {
-                if (position < NOT_ACKED) {
-                    position = NOT_ACKED;
-                }
-            }
-            return position;
-        }
-
         public int getMessageCount(String clientId, String subscriptionName) throws IOException {
             final String subscriptionKey = subscriptionKey(clientId, subscriptionName);
-            final SubscriptionInfo info = lookupSubscription(clientId, subscriptionName);
             indexLock.writeLock().lock();
             try {
                 return pageFile.tx().execute(new Transaction.CallableClosure<Integer, IOException>() {
                     public Integer execute(Transaction tx) throws IOException {
                         StoredDestination sd = getStoredDestination(dest, tx);
-                        Long cursorPos = sd.subscriptionAcks.get(tx, subscriptionKey);
+                        LastAck cursorPos = sd.subscriptionAcks.get(tx, subscriptionKey);
                         if (cursorPos == null) {
                             // The subscription might not exist.
                             return 0;
                         }
-                        cursorPos = resetForSelectors(info, cursorPos);
 
                         int counter = 0;
-                        try {
-                            String selector = info.getSelector();
-                            BooleanExpression selectorExpression = null;
-                            if (selector != null) {
-                                selectorExpression = SelectorParser.parse(selector);
+                        for (Iterator<Entry<Long, HashSet<String>>> iterator =
+                                sd.ackPositions.iterator(tx, cursorPos.lastAckedSequence); iterator.hasNext();) {
+                            Entry<Long, HashSet<String>> entry = iterator.next();
+                            if (entry.getValue().contains(subscriptionKey)) {
+                                counter++;
                             }
-                            sd.orderIndex.resetCursorPosition();
-                            sd.orderIndex.setBatch(tx, extractSequenceId(cursorPos));
-                            for (Iterator<Entry<Long, MessageKeys>> iterator = sd.orderIndex.iterator(tx); iterator
-                                    .hasNext();) {
-                                Entry<Long, MessageKeys> entry = iterator.next();
-                                if (selectorExpression != null) {
-                                    MessageEvaluationContext ctx = new MessageEvaluationContext();
-                                    ctx.setMessageReference(loadMessage(entry.getValue().location));
-                                    if (selectorExpression.matches(ctx)) {
-                                        counter++;
-                                    }
-                                } else {
-                                    counter++;
-                                }
-                            }
-                            sd.orderIndex.resetCursorPosition();
-                        } catch (Exception e) {
-                            throw IOExceptionSupport.create(e);
                         }
                         return counter;
                     }
@@ -787,9 +754,8 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                 pageFile.tx().execute(new Transaction.Closure<Exception>() {
                     public void execute(Transaction tx) throws Exception {
                         StoredDestination sd = getStoredDestination(dest, tx);
-                        Long cursorPos = sd.subscriptionAcks.get(tx, subscriptionKey);
-                        cursorPos = resetForSelectors(info, cursorPos);
-                        sd.orderIndex.setBatch(tx, extractSequenceId(cursorPos));
+                        LastAck cursorPos = sd.subscriptionAcks.get(tx, subscriptionKey);
+                        sd.orderIndex.setBatch(tx, cursorPos);
                         for (Iterator<Entry<Long, MessageKeys>> iterator = sd.orderIndex.iterator(tx); iterator
                                 .hasNext();) {
                             Entry<Long, MessageKeys> entry = iterator.next();
@@ -815,9 +781,12 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                         sd.orderIndex.resetCursorPosition();
                         MessageOrderCursor moc = sd.subscriptionCursors.get(subscriptionKey);
                         if (moc == null) {
-                            Long pos = sd.subscriptionAcks.get(tx, subscriptionKey);
-                            pos = resetForSelectors(info, pos);
-                            sd.orderIndex.setBatch(tx, extractSequenceId(pos));
+                            LastAck pos = sd.subscriptionAcks.get(tx, subscriptionKey);
+                            if (pos == null) {
+                                // sub deleted
+                                return;
+                            }
+                            sd.orderIndex.setBatch(tx, pos);
                             moc = sd.orderIndex.cursor;
                         } else {
                             sd.orderIndex.cursor.sync(moc);
@@ -839,9 +808,6 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                         if (entry != null) {
                             MessageOrderCursor copy = sd.orderIndex.cursor.copy();
                             sd.subscriptionCursors.put(subscriptionKey, copy);
-                            if (LOG.isDebugEnabled()) {
-                                LOG.debug("updated moc: " + copy + ", recovered: " + counter);
-                            }
                         }
                     }
                 });
@@ -928,7 +894,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                         ActiveMQDestination dest = convert(entry.getKey());
                         if (dest.isTopic()) {
                             StoredDestination loadedStore = getStoredDestination(convert(dest), tx);
-                            if (loadedStore.ackPositions.isEmpty()) {
+                            if (loadedStore.subscriptionAcks.isEmpty(tx)) {
                                 isEmptyTopic = true;
                             }
                         }
@@ -994,29 +960,6 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
     // Internal conversion methods.
     // /////////////////////////////////////////////////////////////////
 
-    KahaTransactionInfo createTransactionInfo(TransactionId txid) {
-        if (txid == null) {
-            return null;
-        }
-        KahaTransactionInfo rc = new KahaTransactionInfo();
-
-        if (txid.isLocalTransaction()) {
-            LocalTransactionId t = (LocalTransactionId) txid;
-            KahaLocalTransactionId kahaTxId = new KahaLocalTransactionId();
-            kahaTxId.setConnectionId(t.getConnectionId().getValue());
-            kahaTxId.setTransacitonId(t.getValue());
-            rc.setLocalTransacitonId(kahaTxId);
-        } else {
-            XATransactionId t = (XATransactionId) txid;
-            KahaXATransactionId kahaTxId = new KahaXATransactionId();
-            kahaTxId.setBranchQualifier(new Buffer(t.getBranchQualifier()));
-            kahaTxId.setGlobalTransactionId(new Buffer(t.getGlobalTransactionId()));
-            kahaTxId.setFormatId(t.getFormatId());
-            rc.setXaTransacitonId(kahaTxId);
-        }
-        return rc;
-    }
-
     KahaLocation convert(Location location) {
         KahaLocation rc = new KahaLocation();
         rc.setLogId(location.getDataFileId());
