@@ -27,6 +27,7 @@ import java.util.concurrent.atomic.AtomicInteger;
 import org.apache.activemq.command.Command;
 import org.apache.activemq.command.ConsumerInfo;
 import org.apache.activemq.command.Message;
+import org.apache.activemq.command.RemoveInfo;
 import org.apache.activemq.command.Response;
 import org.apache.activemq.state.ConnectionStateTracker;
 import org.apache.activemq.thread.DefaultThreadPools;
@@ -65,7 +66,7 @@ public class FanoutTransport implements CompositeTransport {
     private final TaskRunner reconnectTask;
     private boolean started;
 
-    private ArrayList<FanoutTransportHandler> transports = new ArrayList<FanoutTransportHandler>();
+    private final ArrayList<FanoutTransportHandler> transports = new ArrayList<FanoutTransportHandler>();
     private int connectedCount;
 
     private int minAckCount = 2;
@@ -73,7 +74,7 @@ public class FanoutTransport implements CompositeTransport {
     private long initialReconnectDelay = 10;
     private long maxReconnectDelay = 1000 * 30;
     private long backOffMultiplier = 2;
-    private boolean useExponentialBackOff = true;
+    private final boolean useExponentialBackOff = true;
     private int maxReconnectAttempts;
     private Exception connectionFailure;
     private FanoutTransportHandler primary;
@@ -89,6 +90,7 @@ public class FanoutTransport implements CompositeTransport {
             this.ackCount = new AtomicInteger(count);
         }
 
+        @Override
         public String toString() {
             return command.getCommandId() + "=" + ackCount.get();
         }
@@ -107,6 +109,7 @@ public class FanoutTransport implements CompositeTransport {
             this.uri = uri;
         }
 
+        @Override
         public void onCommand(Object o) {
             Command command = (Command)o;
             if (command.isResponse()) {
@@ -125,10 +128,11 @@ public class FanoutTransport implements CompositeTransport {
             }
         }
 
+        @Override
         public void onException(IOException error) {
             try {
                 synchronized (reconnectMutex) {
-                    if (transport == null) {
+                    if (transport == null || !transport.isConnected()) {
                         return;
                     }
 
@@ -432,7 +436,8 @@ public class FanoutTransport implements CompositeTransport {
             }
             return ((Message)command).getDestination().isTopic();
         }
-        if (command.getDataStructureType() == ConsumerInfo.DATA_STRUCTURE_TYPE) {
+        if (command.getDataStructureType() == ConsumerInfo.DATA_STRUCTURE_TYPE ||
+                command.getDataStructureType() == RemoveInfo.DATA_STRUCTURE_TYPE) {
             return false;
         }
         return true;
@@ -499,7 +504,7 @@ public class FanoutTransport implements CompositeTransport {
         }
     }
 
-    public void add(URI uris[]) {
+    public void add(boolean reblance,URI uris[]) {
 
         synchronized (reconnectMutex) {
             for (int i = 0; i < uris.length; i++) {
@@ -523,7 +528,7 @@ public class FanoutTransport implements CompositeTransport {
 
     }
 
-    public void remove(URI uris[]) {
+    public void remove(boolean rebalance,URI uris[]) {
 
         synchronized (reconnectMutex) {
             for (int i = 0; i < uris.length; i++) {
@@ -546,9 +551,20 @@ public class FanoutTransport implements CompositeTransport {
     }
     
     public void reconnect(URI uri) throws IOException {
-		add(new URI[]{uri});
+		add(true,new URI[]{uri});
 		
 	}
+    
+    public boolean isReconnectSupported() {
+        return true;
+    }
+
+    public boolean isUpdateURIsSupported() {
+        return true;
+    }
+    public void updateURIs(boolean reblance,URI[] uris) throws IOException {
+        add(reblance,uris);
+    }
 
 
     public String getRemoteAddress() {
