@@ -27,6 +27,8 @@ import java.util.concurrent.atomic.AtomicBoolean;
 
 import javax.jms.JMSException;
 
+import org.apache.activemq.broker.BrokerContext;
+import org.apache.activemq.broker.BrokerContextAware;
 import org.apache.activemq.command.ActiveMQDestination;
 import org.apache.activemq.command.ActiveMQMessage;
 import org.apache.activemq.command.ActiveMQTempQueue;
@@ -58,7 +60,8 @@ import org.apache.activemq.util.IOExceptionSupport;
 import org.apache.activemq.util.IdGenerator;
 import org.apache.activemq.util.IntrospectionSupport;
 import org.apache.activemq.util.LongSequenceGenerator;
-import org.springframework.context.ApplicationContext;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 import org.springframework.context.ApplicationContextAware;
 
 /**
@@ -66,6 +69,8 @@ import org.springframework.context.ApplicationContextAware;
  */
 public class ProtocolConverter {
 
+    private static final Log LOG = LogFactory.getLog(ProtocolConverter.class);
+    
     private static final IdGenerator CONNECTION_ID_GENERATOR = new IdGenerator();
 
     private final ConnectionId connectionId = new ConnectionId(CONNECTION_ID_GENERATOR.generateId());
@@ -82,19 +87,19 @@ public class ProtocolConverter {
     private final ConcurrentHashMap<String, ActiveMQDestination> tempDestinations = new ConcurrentHashMap<String, ActiveMQDestination>();
     private final ConcurrentHashMap<String, String> tempDestinationAmqToStompMap = new ConcurrentHashMap<String, String>();
     private final Map<String, LocalTransactionId> transactions = new ConcurrentHashMap<String, LocalTransactionId>();
-    private final StompTransportFilter transportFilter;
+    private final StompTransport stompTransport;
 
     private final Object commnadIdMutex = new Object();
     private int lastCommandId;
     private final AtomicBoolean connected = new AtomicBoolean(false);
     private final FrameTranslator frameTranslator;
     private final FactoryFinder FRAME_TRANSLATOR_FINDER = new FactoryFinder("META-INF/services/org/apache/activemq/transport/frametranslator/");
-    private final ApplicationContext applicationContext;
+    private final BrokerContext brokerContext;
 
-    public ProtocolConverter(StompTransportFilter stompTransportFilter, FrameTranslator translator, ApplicationContext applicationContext) {
-        this.transportFilter = stompTransportFilter;
+    public ProtocolConverter(StompTransport stompTransport, FrameTranslator translator, BrokerContext brokerContext) {
+        this.stompTransport = stompTransport;
         this.frameTranslator = translator;
-        this.applicationContext = applicationContext;
+        this.brokerContext = brokerContext;
     }
 
     protected int generateCommandId() {
@@ -118,7 +123,7 @@ public class ProtocolConverter {
                         sc.setAction(Stomp.Responses.RECEIPT);
                         sc.setHeaders(new HashMap<String, String>(1));
                         sc.getHeaders().put(Stomp.Headers.Response.RECEIPT_ID, receiptId);
-                        transportFilter.sendToStomp(sc);
+                        stompTransport.sendToStomp(sc);
                     }
                 }
             };
@@ -132,35 +137,35 @@ public class ProtocolConverter {
             command.setResponseRequired(true);
             resposeHandlers.put(Integer.valueOf(command.getCommandId()), handler);
         }
-        transportFilter.sendToActiveMQ(command);
+        stompTransport.sendToActiveMQ(command);
     }
 
     protected void sendToStomp(StompFrame command) throws IOException {
-        transportFilter.sendToStomp(command);
+        stompTransport.sendToStomp(command);
     }
-    
+
     protected FrameTranslator findTranslator(String header) {
-		FrameTranslator translator = frameTranslator;
-		try {
-			if (header != null) {
-				translator = (FrameTranslator) FRAME_TRANSLATOR_FINDER
-						.newInstance(header);
-				if (translator instanceof ApplicationContextAware) {
-					((ApplicationContextAware)translator).setApplicationContext(applicationContext);
-				}
-			}
-		} catch (Exception ignore) {
-			// if anything goes wrong use the default translator
-		}
-		
-		return translator;
-	}
+        FrameTranslator translator = frameTranslator;
+        try {
+            if (header != null) {
+                translator = (FrameTranslator) FRAME_TRANSLATOR_FINDER
+                        .newInstance(header);
+                if (translator instanceof BrokerContextAware) {
+                    ((BrokerContextAware)translator).setBrokerContext(brokerContext);
+                }
+            }
+        } catch (Exception ignore) {
+            // if anything goes wrong use the default translator
+        }
+
+        return translator;
+    }
 
     /**
-	 * Convert a stomp command
-	 * 
-	 * @param command
-	 */
+     * Convert a stomp command
+     *
+     * @param command
+     */
     public void onStompCommand(StompFrame command) throws IOException, JMSException {
         try {
 
@@ -195,12 +200,17 @@ public class ProtocolConverter {
             handleException(e, command);
             // Some protocol errors can cause the connection to get closed.
             if( e.isFatal() ) {
-               getTransportFilter().onException(e);
+               getStompTransport().onException(e);
             }
         }
     }
-    
+
     protected void handleException(Throwable exception, StompFrame command) throws IOException {
+        LOG.warn("Exception occurred processing: \n" + command + ": " + exception.toString());
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Exception detail", exception);
+        }
+
         // Let the stomp client know about any protocol errors.
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
         PrintWriter stream = new PrintWriter(new OutputStreamWriter(baos, "UTF-8"));
@@ -211,10 +221,10 @@ public class ProtocolConverter {
         headers.put(Stomp.Headers.Error.MESSAGE, exception.getMessage());
 
         if (command != null) {
-        	final String receiptId = command.getHeaders().get(Stomp.Headers.RECEIPT_REQUESTED);
-        	if (receiptId != null) {
-        		headers.put(Stomp.Headers.Response.RECEIPT_ID, receiptId);
-        	}
+            final String receiptId = command.getHeaders().get(Stomp.Headers.RECEIPT_REQUESTED);
+            if (receiptId != null) {
+                headers.put(Stomp.Headers.Response.RECEIPT_ID, receiptId);
+            }
         }
 
         StompFrame errorMessage = new StompFrame(Stomp.Responses.ERROR, headers, baos.toByteArray());
@@ -331,7 +341,7 @@ public class ProtocolConverter {
         if (activemqTx == null) {
             throw new ProtocolException("Invalid transaction id: " + stompTx);
         }
-        
+
         for (Iterator<StompSubscription> iter = subscriptionsByConsumerId.values().iterator(); iter.hasNext();) {
             StompSubscription sub = iter.next();
             sub.onStompCommit(activemqTx);
@@ -343,7 +353,7 @@ public class ProtocolConverter {
         tx.setType(TransactionInfo.COMMIT_ONE_PHASE);
 
         sendToActiveMQ(tx, createResponseHandler(command));
-        
+
     }
 
     protected void onStompAbort(StompFrame command) throws ProtocolException {
@@ -362,9 +372,9 @@ public class ProtocolConverter {
         for (Iterator<StompSubscription> iter = subscriptionsByConsumerId.values().iterator(); iter.hasNext();) {
             StompSubscription sub = iter.next();
             try {
-            	sub.onStompAbort(activemqTx);
+                sub.onStompAbort(activemqTx);
             } catch (Exception e) {
-            	throw new ProtocolException("Transaction abort failed", false, e);
+                throw new ProtocolException("Transaction abort failed", false, e);
             }
         }
 
@@ -386,6 +396,11 @@ public class ProtocolConverter {
         String destination = headers.get(Stomp.Headers.Subscribe.DESTINATION);
 
         ActiveMQDestination actualDest = translator.convertDestination(this, destination);
+
+        if (actualDest == null) {
+            throw new ProtocolException("Invalid Destination.");
+        }
+
         ConsumerId id = new ConsumerId(sessionId, consumerIdGenerator.getNextSequenceId());
         ConsumerInfo consumerInfo = new ConsumerInfo(id);
         consumerInfo.setPrefetchSize(1000);
@@ -430,9 +445,9 @@ public class ProtocolConverter {
         if (subscriptionId == null && destination == null) {
             throw new ProtocolException("Must specify the subscriptionId or the destination you are unsubscribing from");
         }
-       
+
         // check if it is a durable subscription
-        String durable = command.getHeaders().get("activemq.subscriptionName"); 
+        String durable = command.getHeaders().get("activemq.subscriptionName");
         if (durable != null) {
             RemoveSubscriptionInfo info = new RemoveSubscriptionInfo();
             info.setClientId(durable);
@@ -455,12 +470,12 @@ public class ProtocolConverter {
                 return;
             }
         }
-       
+
         throw new ProtocolException("No subscription matched.");
     }
 
     ConnectionInfo connectionInfo = new ConnectionInfo();
-    
+
     protected void onStompConnect(final StompFrame command) throws ProtocolException {
 
         if (connected.get()) {
@@ -487,7 +502,7 @@ public class ProtocolConverter {
         connectionInfo.setResponseRequired(true);
         connectionInfo.setUserName(login);
         connectionInfo.setPassword(passcode);
-        connectionInfo.setTransportContext(transportFilter.getPeerCertificates());
+        connectionInfo.setTransportContext(stompTransport.getPeerCertificates());
 
         sendToActiveMQ(connectionInfo, new ResponseHandler() {
             public void onResponse(ProtocolConverter converter, Response response) throws IOException {
@@ -496,7 +511,7 @@ public class ProtocolConverter {
                     // If the connection attempt fails we close the socket.
                     Throwable exception = ((ExceptionResponse)response).getException();
                     handleException(exception, command);
-                    getTransportFilter().onException(IOExceptionSupport.create(exception));
+                    getStompTransport().onException(IOExceptionSupport.create(exception));
                     return;
                 }
 
@@ -506,14 +521,14 @@ public class ProtocolConverter {
                 final ProducerInfo producerInfo = new ProducerInfo(producerId);
                 sendToActiveMQ(producerInfo, new ResponseHandler() {
                     public void onResponse(ProtocolConverter converter, Response response) throws IOException {
-                        
+
                         if (response.isException()) {
                             // If the connection attempt fails we close the socket.
                             Throwable exception = ((ExceptionResponse)response).getException();
                             handleException(exception, command);
-                            getTransportFilter().onException(IOExceptionSupport.create(exception));
+                            getStompTransport().onException(IOExceptionSupport.create(exception));
                         }
-                        
+
                         connected.set(true);
                         HashMap<String, String> responseHeaders = new HashMap<String, String>();
 
@@ -555,7 +570,7 @@ public class ProtocolConverter {
 
     /**
      * Dispatch a ActiveMQ command
-     * 
+     *
      * @param command
      * @throws IOException
      */
@@ -593,39 +608,39 @@ public class ProtocolConverter {
     }
 
     public StompFrame convertMessage(ActiveMQMessage message, boolean ignoreTransformation) throws IOException, JMSException {
-    	if (ignoreTransformation == true) {
-    		return frameTranslator.convertMessage(this, message);
-    	} else {
-    		return findTranslator(message.getStringProperty(Stomp.Headers.TRANSFORMATION)).convertMessage(this, message);
-    	}
+        if (ignoreTransformation == true) {
+            return frameTranslator.convertMessage(this, message);
+        } else {
+            return findTranslator(message.getStringProperty(Stomp.Headers.TRANSFORMATION)).convertMessage(this, message);
+        }
     }
 
-    public StompTransportFilter getTransportFilter() {
-        return transportFilter;
+    public StompTransport getStompTransport() {
+        return stompTransport;
     }
 
-	public ActiveMQDestination createTempQueue(String name) {
+    public ActiveMQDestination createTempQueue(String name) {
         ActiveMQDestination rc = tempDestinations.get(name);
         if( rc == null ) {
             rc = new ActiveMQTempQueue(connectionId, tempDestinationGenerator.getNextSequenceId());
             sendToActiveMQ(new DestinationInfo(connectionId, DestinationInfo.ADD_OPERATION_TYPE, rc), null);
             tempDestinations.put(name, rc);
-        }        
+        }
         return rc;
-	}
+    }
 
-	public ActiveMQDestination createTempTopic(String name) {
+    public ActiveMQDestination createTempTopic(String name) {
         ActiveMQDestination rc = tempDestinations.get(name);
         if( rc == null ) {
             rc = new ActiveMQTempTopic(connectionId, tempDestinationGenerator.getNextSequenceId());
             sendToActiveMQ(new DestinationInfo(connectionId, DestinationInfo.ADD_OPERATION_TYPE, rc), null);
             tempDestinations.put(name, rc);
             tempDestinationAmqToStompMap.put(rc.getQualifiedName(), name);
-        }        
+        }
         return rc;
-	}
+    }
 
-	public String getCreatedTempDestinationName(ActiveMQDestination destination) {
-		return tempDestinationAmqToStompMap.get(destination.getQualifiedName());
-	}
+    public String getCreatedTempDestinationName(ActiveMQDestination destination) {
+        return tempDestinationAmqToStompMap.get(destination.getQualifiedName());
+    }
 }
