@@ -152,24 +152,40 @@ public class JmsMultipleBrokersTestSupport extends CombinationTestSupport {
         maxSetupTime = 8000;
     }
 
-    
-    protected void waitForBridgeFormation() throws Exception {
+
+    protected void waitForBridgeFormation(final int min) throws Exception {
         for (BrokerItem brokerItem : brokers.values()) {
             final BrokerService broker = brokerItem.broker;
-            if (!broker.getNetworkConnectors().isEmpty()) {
-                Wait.waitFor(new Wait.Condition() {
-                    public boolean isSatisified() throws Exception {
-                        return !broker.getNetworkConnectors().get(0).activeBridges().isEmpty();
-                    }});
-            }
+            waitForBridgeFormation(broker, min, 0);
         }
     }
 
+    public boolean waitForBridgeFormation(final BrokerService broker, final int min, final int bridgeIndex) throws Exception {
+        return waitForBridgeFormation(broker, min, bridgeIndex, Wait.MAX_WAIT_MILLIS*2);
+    }
+
+    public boolean waitForBridgeFormation(final BrokerService broker, final int min, final int bridgeIndex, long wait) throws Exception {
+
+        boolean result = false;
+        if (!broker.getNetworkConnectors().isEmpty()) {
+            result = Wait.waitFor(new Wait.Condition() {
+                public boolean isSatisified() throws Exception {
+                    return (broker.getNetworkConnectors().get(bridgeIndex).activeBridges().size() >= min);
+                }}, wait);
+        }
+        return result;
+    }
+
+    protected void waitForBridgeFormation() throws Exception {
+        waitForBridgeFormation(1);
+    }
+
     protected void startAllBrokers() throws Exception {
         Collection<BrokerItem> brokerList = brokers.values();
         for (Iterator<BrokerItem> i = brokerList.iterator(); i.hasNext();) {
             BrokerService broker = i.next().broker;
             broker.start();
+            broker.waitUntilStarted();
         }
 
         Thread.sleep(maxSetupTime);
@@ -517,6 +533,7 @@ public class JmsMultipleBrokersTestSupport extends CombinationTestSupport {
             }
 
             broker.stop();
+            broker.waitUntilStopped();
             consumers.clear();
 
             broker = null;
